# coding=utf-8

from flask import request


def make_cors_headers(allowed_origins=None,
                      allowed_credentials=False,
                      max_age=86400):
    headers = dict()
    headers["Access-Control-Allow-Headers"] = _make_allow_headers()
    headers_options = "OPTIONS, HEAD, POST, PUT, DELETE"
    headers["Access-Control-Allow-Methods"] = headers_options

    if '*' in allowed_origins:
        headers["Access-Control-Allow-Origin"] = '*'
    elif request.headers.get('Origin') in allowed_origins:
        headers["Access-Control-Allow-Origin"] = request.headers['Origin']

    if allowed_credentials:
        headers["Access-Control-Allow-Credentials"] = 'true'

    headers["Access-Control-Max-Age"] = max_age
    return headers


def _make_allow_headers():
    request_allows = request.headers.get(
        "Access-Control-Request-Headers", None)
    if request_allows:
        return request_allows
    else:
        base_set = ["origin", "accept", "content-type", "authorization"]
        return ", ".join(base_set)



# coding=utf-8


DEFAULT_METHODS = {
    'get': 'GET',
    'list': 'GET',
    'create': 'POST',
    'update': 'PUT',
    'delete': 'DELETE'
}

DEFAULT_ENDPOINTS = DEFAULT_METHODS.keys()


def METHOD(methods, name):
    return methods.get(name, 'POST')



# coding=utf-8

from .constants import DEFAULT_METHODS, METHOD
from .response import output_json


class Resource(object):
    _extends_ = {}
    _endpoints_ = {}

    def __init__(self):
        extends = self._extends_
        methods = dict(DEFAULT_METHODS, **extends)
        self._endpoints_ = {name: METHOD(methods, name) for name in dir(self)
                            if not name.startswith('_')}
        for name in dir(self):
            if not name.startswith('_'):
                func = output_json(getattr(self, name))
                setattr(self, name, func)


def extend(**extends):
    def add_extends(cls):
        cls._extends_ = extends
        return cls
    return add_extends



# coding=utf-8


def classfy_dict(data, func):
    match = {}
    not_match = {}

    for key, value in data.items():
        if func(key, value):
            match[key] = value
        else:
            not_match[key] = value

    return match, not_match


def inject_url_rule(app_or_blueprint, rule, methods, endpoint, func):
    methods = methods.split()
    app_or_blueprint.add_url_rule(rule, endpoint=endpoint,
                                  view_func=func, methods=methods)


def auto_complete_rule(rule):
    if not rule.endswith('/'):
        return rule + '/'
    else:
        return rule



# coding=utf-8


def list_restful_apis(*restful_apis):
    for restful_api in restful_apis:
        list_restful_api(restful_api)


def list_restful_api(restful_api):
    for resource, routes in restful_api.routes.items():
        print(resource)
        for rule, methods, func in routes:
            print(route_string(rule, methods, func))


def list_common_apis(apis):
    for rule, methods, func in apis.routes:
        print(route_string(rule, methods, func))


def route_string(rule, methods, func):
    return "{} {} {}".format(rule, methods, func.__name__)



# coding=utf-8

import json
import functools

from flask import make_response

from .headers import make_cors_headers


def output_json(func):
    @functools.wraps(func)
    def decorate(*args, **kwargs):
        result = func(*args, **kwargs)
        data, status_code = load_result(result)
        return make_json_response(data, status_code or 200)

    return decorate


def load_result(result):
    if isinstance(result, tuple) and len(result) == 2:
        return result
    else:
        return result, 200


def make_json_response(data, status_code, headers=None):
    headers = headers or {}
    DEFAULT_HEADERS = {"Content-Type": "application/json"}
    headers.update(DEFAULT_HEADERS)
    headers.update(make_cors_headers(allowed_origins='*'))
    data = json.dumps(data)
    return make_response(data, status_code, headers)



#!/usr/bin/env python

"""
Flask-REST4
-------------
Elegant RESTful API for your Flask apps.
"""
from setuptools import setup, find_packages


setup(
    name='flask_rest4',
    version='0.1.7',
    packages=find_packages(),
    url='https://github.com/squirrelmajik/flask_rest4',
    license='See License',
    author='majik',
    author_email='me@yamajik.com',
    description='Elegant RESTful API for your Flask apps.',
    long_description=__doc__,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# coding=utf-8

from .utils import inject_url_rule, auto_complete_rule
from .resource import output_json


class Api(object):
    def __init__(self, app_or_blueprint):
        self.app = app_or_blueprint
        self.routes = []

    def route(self, rule, methods):
        def decorator(func):
            func = output_json(func)
            self.register_api(rule, methods, func)
            return func
        return decorator

    def register_api(self, rule, methods, func):
        endpoint = func.__name__
        rule = auto_complete_rule(rule)
        inject_url_rule(self.app, rule, methods, endpoint, func)
        route = (rule, methods, func)
        self.routes.append(route)



# coding=utf-8

import re

from .constants import DEFAULT_ENDPOINTS
from .utils import classfy_dict, inject_url_rule, auto_complete_rule


class RESTful(object):
    def __init__(self, app_or_blueprint):
        self.app = app_or_blueprint
        self.routes = {}

    def route(self, rule):
        def decorator(Resource):
            self.register_resource(rule, Resource)
            return Resource
        return decorator

    def register_resource(self, rule, Resource):
        batch_rule, item_rule = _analyze_rule(rule)

        def register_resource(Resource):
            resource = Resource()
            resource_name = type(resource).__name__

            def is_batch_endpoint(func_name, methods):
                batch_signs = ['batch', 'list', 'create']
                return any(sign in func_name for sign in batch_signs)

            batch_endpoints, item_endpoints = classfy_dict(
                resource._endpoints_, is_batch_endpoint)

            def reject_rule(func_name, methods, is_batch):
                rule = batch_rule if is_batch else item_rule
                if func_name not in DEFAULT_ENDPOINTS:
                    rule += (func_name + '/')
                func = getattr(resource, func_name)
                endpoint = r'{}_{}'.format(resource_name, func_name)
                inject_url_rule(self.app, rule, methods, endpoint, func)
                return rule, methods, func

            batch_routes = [reject_rule(func_name, methods, True)
                            for func_name, methods in batch_endpoints.items()]
            item_routes = [reject_rule(func_name, methods, False)
                           for func_name, methods in item_endpoints.items()]
            self.routes[resource_name] = batch_routes + item_routes

        return register_resource(Resource)


def _analyze_rule(rule):
    rule = auto_complete_rule(rule)

    match = re.match(r'^(.*)<\w+?>/$', rule)
    if not match:
        raise Exception

    batch_rule = match.groups()[0]
    item_rule = rule
    return batch_rule, item_rule
